//  Demonstrates use of independent processes to draw a series of bouncing
//  blobs. Written by
//  Josh Turpen (snarfy@goodnet.com) and
//  Sengan Short(sengan.short@durham.ac.uk)
//  Minor modification by Paolo De Marino (pdemarin@mbx.idn.it)
//  This demo file is FREEWARE - you can do whatever you want with it.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING. If not, write to the
//  Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "lwp.h"
#include <dpmi.h>
#include <dos.h>
#include <stdlib.h>
#include <go32.h>
#include <sys/farptr.h>
#include <time.h>

#define BLOB_COUNT 10  /* 10 threads */
volatile int plot_count = 0;
void putpixel(int x, int y, char col);
void blob_proc(void);
void waitVBL(void);

int main()
{
        int i;
        int ids[BLOB_COUNT];
        __dpmi_regs regs;
        regs.x.ax = 0x13;
        srandom(time(0));
        __dpmi_int(0x10, &regs);
        lwp_init(0,128);
        _farsetsel(_dos_ds);
        for(i=0;i<=BLOB_COUNT;i++)
                {
                ids[i] = lwp_spawn(blob_proc, 4096);
                }
        while(!kbhit())
                {
                if(plot_count == BLOB_COUNT)
                        {
                        plot_count = 0;
                        }
                plot_count++;
                }
        for(i=0;i<=BLOB_COUNT;i++)
                {
                lwp_kill(ids[i]);
                }
        lwp_thread_disable();
        getch();
        regs.x.ax = 0x3;
        __dpmi_int(0x10, &regs);
        lwp_thread_enable();
        return(0);
}

void blob_proc(void)
{
        int x, y, xspeed,yspeed;
        char col;
        lwp_thread_disable();
        x = random() & 319;
        y = random() & 199;
        col = random() & 0xF0;
        xspeed = random() & 1;
        yspeed = random() & 1;
        lwp_thread_enable();
        while(1)
                {
                putpixel(x,y,0);
                plot_count++;
                x+=xspeed;
                if(x>=319)
                        {
                        xspeed=-xspeed;
                        x = 319;
                        }
                if(x<=0)
                        {
                        xspeed=-xspeed;
                        x = 0;
                        }
                y+=yspeed;
                if(y>=199)
                        {
                        yspeed=-yspeed;
                        y=199;
                        }
                if(y<=0)
                        {
                        yspeed=-yspeed;
                        y=0;
                        }
                putpixel(x,y,col);
                }
}

void putpixel(int x, int y, char col)
{
        lwp_thread_disable();
        _farnspokeb(0xA0000+x+(y<<6)+(y<<8), col);
        lwp_thread_enable();
}

void waitVBL(void)
{
        lwp_thread_disable();
        while(!(inportb(0x3DA) & 8));
        while(inportb(0x3DA) & 8);
        lwp_thread_enable();
}
